/*
 * download.c
 * 2/16/90
 * (c) 1989, 1990 by Stephen A. Ness.
 * Usage: download
 * Download MIDI information from D-10 to files.
 * Written on the fly, cleaned up later, untested since; might be partly flakey.
 */

#include <osbind.h>
#include <bios.h>
#include <stdio.h>
#include "roland.h"
#include "d10.h"
#include "midimsg.h"

#define	area(a0, a1, a2)	addr[0] = (a0); addr[1] = (a1); addr[2] = (a2)
#define	len(l0, l1, l2)		addr[3] = (l0); addr[4] = (l1); addr[5] = (l2)

extern	void	bump();
extern	void	d10_dt1();
extern	void	d10_rq1();
extern	void	fatal();
extern	void	save();

/* Globals. */
#define	BSIZE	1024
unsigned char	addr[6];
unsigned char	buf[BSIZE];
unsigned int	length;
unsigned char	name[128];

main(argc, argv) int argc; char *argv[];
{
	while (Bconstat(BC_MID) != 0)
		Bconin(BC_MID);			/* clear pending MIDI input */

#if	0
	/* Download 64 internal rhythm tones. */
	/* I don't know how to do this noninteractively. */
	printf("Must be in multitimberal mode, Part 1 = Ch 1.\n");
	area(4, 0, 0);				/* Part 1 tone temp area: 04 00 00 */
	len(0, 1, 0x76);			/* Length: 00 01 76 */
	for (i = 0; i < 64; i++) {
		printf("Edit ch 1 tone to r%02d and type <return>\n", i+1);
		getchar();
		d10_rq1("rtone%02d", "Internal rhythm tone %d", i);
	}
#endif

#if	0
	/* Download 128 internal tones. */
	printf("Must be in multitimberal mode, Part 1 = Ch 1.\n");
	area(4, 0, 0);				/* Part 1 tone temp area: 04 00 00 */
	len(0, 1, 0x76);			/* Length: 00 01 76 */
	for (i = 0; i < 128; i++) {
		Bconout(BC_MID, 0xC0);
		Bconout(BC_MID, i);
		d10_rq1("itone%03d", "Internal tone %d", i);
	}
#endif

#if	0
	/* Download a temporary tone from tone temp 1. */
	/* Useful after entering a tone by hand. */
	area(4, 0, 0);				/* Part 1 tone temp area: 04 00 00 */
	len(0, 1, 0x76);			/* Length: 00 01 76 */
	d10_rq1("temptone", "Temporary tone %d", 1);
#endif

#if	0
	/* Download 128 timbres. */
	area(5, 0, 0);				/* Timbre 0: 05 00 00 */
	len(0, 0, 8);				/* Length: 00 00 08 */
	for (i = 0; i < 128; i++) {
		d10_rq1("timbr%03d", "Timbre %d", i);
		bump(0, 0, 8);
	}
#endif

#if	0
	/* Download 128 patches. */
	area(7, 0, 0);				/* Patch 0: 07 00 00 */
	len(0, 0, 0x26);			/* Length: 00 00 26 */
	for (i = 0; i < 128; i++) {
		d10_rq1("patch%03d", "Patch %d", i);
		bump(0, 0, 0x26);
	}
#endif

#if	0
	/* Download 64 tones. */
	area(8, 0,  0);				/* Tone 0: 08 00 00 */
	len(0, 1, 0x76);			/* Length: 00 01 76 */
	for (i = 0; i < 64; i++) {
		d10_rq1("tone%02d", "Tone %d", i);
		bump(0, 2, 0);
	}
#endif

#if	0
	/* Download 85 rhythm setups. */
	area(9, 0, 0);				/* Setup 0: 09 00 00 */
	len(0, 0, 4);				/* Length: 00 00 04 */
	for (i = 0; i < 85; i++) {
		d10_rq1("rsetup%02d", "Rhythm setup %d", i);
		bump(0, 0, 4);
	}
#endif

#if	0
	/* Download 32 rhythm patterns, in three parts each. */
	/* The patterns are too long to DT1 in a single block. */
	area(0x0A, 0, 0);			/* Pattern 0: 0A 00 00 */
	for (i = 0; i < 32; i++) {
		len(0, 2, 0);			/* Parts a,b length: 00 02 00 */
		d10_rq1("patt%02da", "Rhythm pattern %da", i);
		bump(0, 2, 0);
		d10_rq1("patt%02db", "Rhythm pattern %db", i);
		bump(0, 2, 0);
		len(0, 0, 0x4C);		/* Part c length: 00 00 4C */
		d10_rq1("%02dc", "Rhythm pattern %dc", i);
		bump(0, 0, 0x4C);
	}
#endif

#if	0
	/* Download rhythm track. */
	/* The track is too long to DT1 in a single block. */
	area(0x0C, 0, 0);			/* Track: 0C 00 00 */
	len(0, 2, 0);				/* Length1: 00 02 00 */
	d10_rq1("rtrack%d", "Rhythm track part %d", 1);
	bump(0, 2, 0);
	len(0, 1, 0x76);			/* Length2: 00 01 76 */
	d10_rq1("rtrack%d", "Rhythm track part %d", 2);
#endif

#if	1
	/* Download system area. */
	area(0x10, 0, 0);			/* System: 10 00 00 */
	len(0, 0, 0x32);			/* Length: 00 00 32 */
	d10_rq1("system", "System area", 0);
#endif

	exit(0);
}

/*
 * Bump the data transfer address by the given length.
 */
void
bump(l0, l1, l2) unsigned int l0, l1, l2;
{
	unsigned long l;

	l = (addr[0] << 14) | (addr[1] << 7) | addr[2];
	l += (l0 << 14) | (l1 << 7) | l2;
	addr[0] = (l >> 14) & 0x7F;
	addr[1] = (l >> 7) & 0x7F;
	addr[2] = l & 0x7F;
}

/*
 * Read a DT1 system exclusive message.
 * Typing any character aborts.
 * The address starts at &buf[5], the data at &buf[8].
 * Die on failure.
 */
void
d10_dt1()
{
	register unsigned char *bp, *bpend;
	register unsigned int c;

	bp = buf;
	while (Bconstat(BC_CON) == 0) {
		if (Bconstat(BC_MID) != 0) {
			c = Bconin(BC_MID) & 0xFF;
			if (c == 0xF8)
				continue;		/* ignore TimingClock */
			else if (c == 0xFE)
				continue;		/* ignore active sensing */
			*bp++ = c;
			if (bp >= &buf[BSIZE]) {
				printf("output buffer full\n");
				break;
			} else if (c == EOX)
				break;
		}
	}
	if (Bconstat(BC_CON) != 0)
		Bconin(BC_CON);

	bpend = --bp;
	if (buf[0] != SysEx
	 || buf[1] != M_ID_ROLAND
	 || buf[2] != DEV_ID_MC
	 || buf[3] != MOD_ID_D10)
		fatal("bad header %x %x %x %x", buf[0], buf[1], buf[2], buf[3]);
	if (buf[4] != DT1)
		fatal("bad command %x", buf[4]);
	if (*bpend != EOX)
		fatal("%x not EOX", *bpend);
	for (c = 0, bp = &buf[5]; bp < bpend; )
		c += *bp++;
	if ((c & 0x7F) != 0)
		fatal("bad checksum\n");
	length = (addr[3] << 14) | (addr[4] << 7) | addr[5];
	if ((c = bpend - &buf[8] - 1) != length)
		fatal("DT1: expected %d bytes, got %d\n", length, c);
}

/*
 * Send a D-10 RQ1 message and read the response.
 * The address and length are passed in global 'addr'.
 * Save the result in a file.
 */
void
d10_rq1(namefmt, fmt, n) char *namefmt, *fmt; int n;
{
	register unsigned char *bp, *xp;
	register int csum;
	static unsigned char xbuf[5 + 6 + 1 + 1] =	/* hdr+addr+data+csum+EOX */
		{ SysEx, M_ID_ROLAND, DEV_ID_MC, MOD_ID_D10, RQ1, 0, 0, 0, 0, 0, 0, 0, EOX };

	sprintf(name, namefmt, n);
	printf(fmt, n);
	printf(" from %02x %02x %02x:\n", addr[0], addr[1], addr[2]);
	csum = 0;
	for (bp = &addr[0], xp = &xbuf[5]; bp < &addr[6]; )
		csum += (*xp++ = *bp++);
	*xp++ = (-csum) & 0x7F;
	Midiws(sizeof(xbuf)-1, &xbuf[0]);
	d10_dt1();
	save(&buf[8]);
}

/*
 * Cry and die.
 */
void
fatal(args) char *args;
{
	fprintf(stderr, "download: %r\n", &args);
	exit(1);
}

/* 
 * Save data to a file.
 * Global 'length' gives the file length.
 * Global 'name' gives the file name.
 */
void
save(buf) unsigned char *buf;
{
	register FILE *ofp;

	if ((ofp = fopen(name, "wb")) == NULL)
		fatal("cannot open output file \"%s\"", name);
	else if (fwrite(buf, 1, length, ofp) != length)
		fatal("write error");
	else if (fclose(ofp) == EOF)
		fatal("cannot close output file \"%s\"", name);
}

/* end of download.c */
