/*
 * fixoff.c
 * 12/21/89
 * Filter AllNotesOff from a MIDI multitrack saved file.
 */

#include <stdio.h>
#include "midi.h"
#include "midimsg.h"

#define	T_SIZE	32000		/* track buffer size */
unsigned char buf[T_SIZE];

extern void fatal();

main(argc, argv) int argc; char *argv[];
{
	int i;
	FILE *ifp, *ofp;
	TRACK t;
	unsigned char *icp, *ocp;
	register int tflag;

	if (argc != 3) {
		fprintf(stderr, "Usage: fixoff oldfile newfile\n");
		exit(1);
	}
	if ((ifp = fopen(argv[1], "rb")) == NULL)
		fatal("cannot open input file \"%s\"", argv[1]);
	if ((ofp = fopen(argv[2], "wb")) == NULL)
		fatal("cannot open output file \"%s\"", argv[2]);
	if (fread(&i, sizeof(int), 1, ifp) != 1)
		fatal("magic number read error");
	if (i != MT_MAGIC)
		fatal("\"%s\" is not a multitrack MIDI data file", argv[1]);
	if (fwrite(&i, sizeof(int), 1, ofp) != 1)
		fatal("magic number write error");
	while (fread(&i, sizeof(int), 1, ifp) == 1) {
		if (fwrite(&i, sizeof(int), 1, ofp) != 1)
			fatal("track number write error");
		if (fread(&t, sizeof(TRACK), 1, ifp) != 1)
			fatal("track read error");
		fprintf(stderr, "track %d: oldsize=%ld", i, t.t_size);
		if (t.t_size > T_SIZE)
			fatal("track too big");
		if (fread(&buf[0], 1, (int)t.t_size, ifp) != (int)t.t_size)
			fatal("track data read error");
		tflag = timed(&t);		
		for (icp = ocp = &buf[0]; icp < &buf[t.t_size]; ) {
			if (tflag) {
				if (Msg(*(icp+3)) == CtrlChange
				 && *(icp+4) == AllNotesOff
				 && *(icp+5) == 0)
					icp += 6;
				else {
					*ocp++ = *icp++;
					*ocp++ = *icp++;
					*ocp++ = *icp++;
					*ocp++ = *icp++;
					*ocp++ = *icp++;
					*ocp++ = *icp++;
				}
			} else {
				if (Msg(*icp) == CtrlChange
				 && *(icp+1) == AllNotesOff
				 && *(icp+2) == 0)
					icp += 3;
				else
					*ocp++ = *icp++;
			}
		}
		t.t_size = i = (int)(ocp - &buf[0]);
		fprintf(stderr, " newsize=%d\n", i);
		if (fwrite(&t, sizeof(TRACK), 1, ofp) != 1)
			fatal("track write error");
		if (fwrite(&buf[0], 1, i, ofp) != i)
			fatal("track data write error");
		
	}
	if (!feof(ifp))
		fatal("track number read error");
	exit(0);
}

void
fatal(s) char *s;
{
	fprintf(stderr, "fixoff: %r\n", &s);
	exit(1);
}

/* end of fixoff.c */

