/*
 * get.c
 * 6/16/89
 * Usage: get [ -t ] [ file ]
 * Option:
 *	-t	Time MIDI input data.
 * Read MIDI input.
 * If no 'file' specified, dump MIDI input to the console.
 * Otherwise, write it to the given 'file'.
 */

#include <osbind.h>
#include <bios.h>
#include <stdio.h>
#include "timer.h"

#define	SIZE		50000L
#define	MAX_UINT	((unsigned)65535L)

extern	char *	lmalloc();

extern	void	binary_save();

/* Globals. */
FILE *		ofp;			/* output FILE			*/
unsigned char *	outbuf;			/* output buffer		*/
unsigned char *	outbufend;		/* end of output buffer		*/
int		tflag;			/* timer flag			*/

main(argc, argv) int argc; char *argv[];
{
	register unsigned int c;
	register unsigned char *bp;

	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 't') {
		++tflag;
		--argc;
		++argv;
	}
	if (argc > 2) {
		fprintf(stderr, "Usage:\tget [ -t ] [ file ]\n");
		exit(1);
	}
	if (argc == 2) {
		if ((ofp = fopen(argv[1], "wb")) == NULL) {
			fprintf(stderr, "cannot open output file \"%s\"\n", argv[1]);
			exit(1);
		}
		if ((outbuf = bp = lmalloc(SIZE)) == NULL) {
			fprintf(stderr, "cannot allocate output buffer\n");
			exit(1);
		}
		outbufend = bp + SIZE;
	} else
		bp = NULL;

	/* Initialize. */
	while (Bconstat(BC_MID) != 0)
		Bconin(BC_MID);			/* clear pending MIDI input */
	if (bp != NULL) {
		printf("Type any character to begin.\n");
		Bconin(BC_CON);
	}
	printf("Type any character to quit.\n");
	if (tflag)
		timerA_enab();

	/* Capture until character typed. */
	while (Bconstat(BC_CON) == 0) {
		if (Bconstat(BC_MID) != 0) {
			c = Bconin(BC_MID) & 0xFF;
			if (c == 0xFE)
				continue;		/* ignore active sensing */
			if (bp == NULL) {
				if (tflag)
					printf("%ld ", timerA);
				printf("%02x\n", c);
			} else {
				if (tflag) {
					*(long *)bp = (timerA << 8) | (unsigned long) c;
					bp += sizeof(long);
				} else
					*bp++ = c;
				if (bp >= outbufend) {
					fprintf(stderr, "output buffer full\n");
					break;
				}
			}
		}
	}
	Bconin(BC_CON);

	/* Cleanup. */
	if (tflag)
		timerA_disab();
	if (ofp != NULL) {
		outbufend = bp;
		binary_save();
	}
	exit(0);
}

/* 
 * Save output data (from 'outbuf' to 'outbufp-1') in binary.
 */
void
binary_save()
{
	register long size;
	register unsigned int usize;

	for (size = outbufend - outbuf; size > 0L; ) {
		usize = (size > MAX_UINT) ? MAX_UINT : size;
		if (fwrite(outbuf, 1, usize, ofp) != usize) {
			fprintf(stderr, "write errorn\n");
			exit(1);
		}
		outbuf += usize;
		size -= usize;
	}
	if (fclose(ofp) == EOF) {
		fprintf(stderr, "cannot close output file\n");
		exit(1);
	}
}
	
