/*
 * midi.h
 * (c) 1989 by Stephen A. Ness.
 */

/* Manifest constants. */
#define	VERSION		"V1.7"
#define	ROLAND_D10	1			/* Roland D-10 specific		*/

#define	BUFLEN		128			/* interactive input buf length */
#define	MAX_UINT	((unsigned int)65535L)	/* should be in <limits.h>	*/
#define	MT_MAGIC	0xBAC			/* multitrack file magic number	*/
#define	NARGS		10			/* interactive command args	*/
#define	NTRACKS		9			/* max number of playback TRACKs */
#define	PROMPT		"\nmidi> "		/* interactive input prompt	*/
#define	ST_MAGIC	0xBACC			/* single track file magic number */

/* Time ticks. */
typedef	unsigned long	TIME;
#define	etime(bp)	((((TIME)((bp)[0])) << 16) | ((bp)[1] << 8) | (bp)[2])

/* MIDI channel bitmap. */
typedef	unsigned int	CHANNEL;
#define	ALLCHANS	((CHANNEL)0xFFFF)

/* Recording and playback tracks. */
typedef	struct	track {
	unsigned int	t_flags;
	CHANNEL		t_chan;
	unsigned long	t_size;
	TIME		t_time;
	unsigned char *	t_buf;
} TRACK;
#define	T_INUSE		0x01
#define	T_PLAY		0x02
#define	T_TIMED		0x04
#define	inuse(tp)	(((tp)->t_flags & T_INUSE) != 0)
#define	play(tp)	(((tp)->t_flags & T_PLAY) != 0)
#define	timed(tp)	(((tp)->t_flags & T_TIMED) != 0)
#define	t_end(tp)	((tp)->t_buf + (tp)->t_size)

/* Playback event list. */
typedef	struct	event {
	TIME		e_time;			/* event time			*/
	unsigned char *	e_bp;			/* track buffer pointer		*/
	unsigned char *	e_end;			/* end of track buffer		*/
} EVENT;

/*
 * This source may be compiled under ANSI-compiliant or pre-ANSI compilers.
 */
#if	__STDC__
#define	Void		void
#define	Of(args)	args
#else
#define	Void		char			/* use Void * as generic pointer */
#define	Of(args)	()
#endif

/* midi.c globals */
extern	int		capture;
extern	CHANNEL		chanmap[16];
extern	char		cmdbuf[BUFLEN];
extern	EVENT		event[NTRACKS];
extern	int		fflag;
extern	char *		filename;
extern	int		iargc;
extern	char *		iargv[NARGS];
extern	unsigned long	memsize;
extern	unsigned int	mmbeats;
extern	unsigned int	mmmeasures;
extern	int		mmticks;
extern	TIME		now;
extern	CHANNEL		pchannels;
extern	int		ptracks;
extern	char *		tempfile;
extern	int		tflag;
extern	FILE *		tfp;
extern	TRACK		track[NTRACKS+1];
extern	int		vflag;

/* midi.c functions */
extern	int		main		Of( (int argc, char **argv) );
extern	void		allnotesoff	Of( (CHANNEL chan) );
extern	int		alter		Of( (void) );
extern	int		begin		Of( (void) );
extern	void		capcheck	Of( (TRACK *tp) );
extern	int		captrack	Of( (int i) );
extern	int		control		Of( (void) );
extern	void		dump		Of( (void) );
extern	int		erase		Of( (void) );
extern	int		erase_track	Of( (TRACK *tp) );
extern	int		error		Of( (char *s, ...) );
extern	int		event_compare	Of( (EVENT *ep1, EVENT *ep2) );
extern	int		event_setup	Of( (TIME start, TIME stop) );
extern	void		fatal		Of( (char *s, ...) );
extern	unsigned char *	findtime	Of( (TRACK *tp, TIME t) );
extern	int		getvoice	Of( (void) );
extern	void		initialize	Of( (void) );
extern	int		join		Of( (void) );
extern	void		lmemcpy		Of( (char *p1, char *p2, unsigned long size) );
extern	void		metronome	Of( (void) );
extern	void		midi_flush	Of( (void) );
extern	int		mustbe		Of( (int min, int max) );
extern	int		number		Of( (void) );
extern	int		parse		Of( (void) );
extern	int		playback	Of( (int flag) );
extern	void		printtime	Of( (TIME t) );
extern	int		quantize	Of( (void) );
extern	int		quant_track	Of( (TRACK *tp, unsigned q, unsigned d, TIME start, TIME stop) );
extern	int		record		Of( (void) );
extern	void		status		Of( (int flag) );
extern	TIME		time		Of( (int n) );
extern	int		time_compare	Of( (unsigned char *p1, unsigned char *p2) );
extern	TRACK *		trackid		Of( (int n, int flag) );
extern	void		transform	Of( (void) );
extern	int		untimed		Of( (TRACK *tp, unsigned char *buf, unsigned n, unsigned chan) );
#if	ROLAND_D10
extern	int		window		Of( (void) );
#endif
extern	Void *		xalloc		Of( (unsigned long size) );

/* midifile.c functions */
extern	int		load		Of( (int flag) );
extern	int		load1		Of( (TRACK *tp, FILE *ifp) );
extern	int		save		Of( (void) );
extern	int		save1		Of( (TRACK *tp, FILE *ofp) );

/* midihack.c functions */
extern	int		hack		Of( (void) );

/* midimsg.c functions */
extern	unsigned char *	msg_print	Of( (unsigned char *bp) );

/* libc.a functions */
extern	long	atol();
extern	char *	lmalloc();
extern	char *	lrealloc();
extern	char *	memcpy();
extern	char *	strchr();
extern	char *	tempnam();

/* end of midi.h */
