/*
 * midihack.c
 * (c) 1989 by Stephen A. Ness.
 * Special-purpose MIDI hacks.
 */

#include <stdio.h>
#include "midi.h"
#include "midimsg.h"

/*
 * Process a track with a special purpose MIDI hack.
 * Interactive usage:
 *	x pl track	Set track pan by pitch, left==low right==high
 *	x pr track	Set track pan by pitch, left==high right==low
 */
int
hack()
{
	register unsigned char *bp, *nbp, *bpend;
	unsigned long size;
	TRACK *tp;
	int flag, min, max, range, opan, pan;

	if ((tp = trackid(2, 1)) == NULL)
		return;
	if (iargv[1][0] == 'p' && iargv[1][1] == 'l')
		flag = 1;
	else if (iargv[1][0] == 'p' && iargv[1][1] == 'r')
		flag = -1;
	else
		return error("must specify \"pl\" or \"pr\"");
	
	/* Find the range of pitches used by the track. */
	min = 127;
	max = 0;
	bpend = t_end(tp);
	for (bp = tp->t_buf; bp < bpend; bp += 6)
		if (Msg(bp[3]) == NoteOn && bp[5] != 0) {
			if (bp[4] < min)
				min = bp[4];
			if (bp[4] > max)
				max = bp[4];
		}
	if ((range = max - min) <= 0)
		return error("no pitch range");

	/* Build a new track with pan commands added. */
	size = 0L;
	opan = -1;
	nbp = track[0].t_buf;
	bp = tp->t_buf;
	while (bp < bpend) {
		if (Msg(bp[3]) == NoteOn && bp[5] != 0) {
			pan = (bp[4] - min) * 127 / range;
			if (flag == -1)
				pan = 127 - pan;
			if (opan != pan) {
				size += 6;
				if (size > memsize)
					return error("track buffer overflow");
				*nbp++ = bp[0];
				*nbp++ = bp[1];
				*nbp++ = bp[2];
				*nbp++ = CtrlChange | Chan(bp[3]);
				*nbp++ = Pan;
				*nbp++ = opan = pan;
			}
		}
		/* Copy the command. */
		size += 6;
		if (size > memsize)
			return error("track buffer overflow");
		memcpy(nbp, bp, 6);
		nbp += 6;
		bp += 6;
	}

	/* Copy back the new track. */
	free(tp->t_buf);
	tp->t_buf = xalloc(size);
	tp->t_size = size;
	lmemcpy(tp->t_buf, track[0].t_buf, size);
}

/* end of midihack.c */
