/*
 * midimsg.h
 * (c) 1989 by Stephen A. Ness.
 * Midi message information.
 * Reference: Michael Boom, "Music Through Midi", Appendix A.
 */

/* Classification macros. */
#define	isData(b)	(((b) & 0x80) == 0)
#define	isStatus(b)	(((b) & 0x80) != 0)
#define	isSystem(b)	(((b) & 0xF0) == 0xF0)
#define	isSysEx(b)	((b) == 0xF0)
#define	isSysComm(b)	(((b) & 0xF8) == 0xF0 && !isSysEx(b))
#define	isSysReal(b)	(((b) & 0xF8) == 0xF8)
#define	isChan(n)	(0 <= (n) && (n) <= 15)
#define	Msg(b)		((b) & 0xF0)
#define	Chan(b)		((b) & 0x0F)
#define	Value(lsb, msb)	(((msb) << 15) | (lsb))

/* Channel voice messages. */
#define	NoteOff		0x80		/* NoteOff|chan pitch velocity	*/
#define	NoteOn		0x90		/* NoteOn|chan pitch velocity	*/
#define	PolyKeyPr	0xA0		/* PolyKeyPr|chan pitch press	*/
#define	CtrlChange	0xB0		/* CtrlChange|chan cont value	*/
#define	ProgChange	0xC0		/* ProgChange|chan program	*/
#define	ChanPr		0xD0		/* ChanPr|chan pressure		*/
#define	PitchBend	0xE0		/* PitchBend|chan lsb msb	*/

/* Channel mode messages. */
/* These are "cont" field of "CtrlChange|Chan cont value". */
#define	Modulation	0x01
#define	Volume		0x07
#define	Pan		0x0A
#define	Expression	0x0B
#define	Sustain		0x40
#define	LocalCtrl	0x7A
#define	AllNotesOff	0x7B		/* value==0 */
#define	OmniModeOff	0x7C		/* value==0 */
#define	OmniModeOn	0x7D		/* value==0 */
#define	MonoModeOn	0x7E		/* value==0 */
#define	PolyModeOn	0x7F		/* value==0 */

/* System exclusive messages. */
/* Variable length, delimited by EOX. */
#define	SysEx		0xF0		/* SysEx id data ... EOX	*/

/* System common messages. */
#define	SongPosPtr	0xF2		/* SongPosPtr lsb msb		*/
#define	SongSelect	0xF3		/* SongSelect song		*/
#define	TuneRequest	0xF6		/* TuneRequest			*/
#define	EOX		0xF7		/* EOX				*/

/* System real-time messages. */
/* All are single-byte messages. */
#define	TICKSPERBEAT	24		/* TimingClock ticks per beat	*/
#define	TimingClock	0xF8
#define	Start		0xFA
#define	Continue	0xFB
#define	Stop		0xFC
#define	ActiveSensing	0xFE
#define	SysReset	0xFF

/*
 * Midi pitch.
 * A midi pitch is a number between 0 and 127.
 * 0 represents C-2 (C five octaves below middle C,
 * 60 represents C3 (middle C),
 * 127 represents G8 (G five and a half octaves above middle C).
 * Reference: Michael Boom, "Music Through Midi", pp. 24-25, 77.
 */
#define	isPitch(p)	(0 <= (p) && (p) <= 127)
#define	note(p)		((p) % _OCTAVE)
#define	octave(p)	((p) / _OCTAVE - 2)
#define	_OCTAVE		12

/* end of midimsg.h */
