/*
 * put.c
 * 6/16/89
 * Usage: put [ -t ] file
 * Option:
 *	-t	MIDI input data is timed.
 * Read MIDI input.
 * Read MIDI input data from 'file' and write it to MIDI.
 */

#include <osbind.h>
#include <bios.h>
#include <stdio.h>
#include "timer.h"

#define	MAX_UINT	((unsigned)65535L)

extern	char *	lmalloc();

extern	void	binary_load();
extern	void	fatal();

/* Globals. */
FILE *		ifp;			/* input FILE			*/
unsigned char *	inbuf;			/* input buffer			*/
unsigned char *	inbufend;		/* end of input buffer		*/
int		tflag;			/* timer flag			*/

main(argc, argv) int argc; char *argv[];
{
	register unsigned int c;
	register unsigned char *bp;
	register unsigned long l;

	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 't') {
		++tflag;
		--argc;
		++argv;
	}
	if (argc != 2) {
		fprintf(stderr, "Usage:\tput [ -t ] file\n");
		exit(1);
	}
	if ((ifp = fopen(argv[1], "rb")) == NULL) {
		fprintf(stderr, "cannot open input file \"%s\"\n", argv[1]);
		exit(1);
	}
	binary_load();

	/* Initialize. */
	printf("Type any character to quit.\n");
	if (tflag)
		timerA_enab();

	/* Playback until character typed or finished. */
	for (bp = inbuf; Bconstat(BC_CON) == 0 && bp < inbufend; ) {
		if (tflag) {
			l = *(long *)bp;
			if ((l >> 8) > timerA)
				continue;
			c = l & 0xFF;
			Bconout(BC_MID, c);
			bp += sizeof(long);
		} else
			Bconout(BC_MID, *bp++);
	}
	if (Bconstat(BC_CON) != 0)
		Bconin(BC_CON);

	/* Cleanup. */
	if (tflag)
		timerA_disab();
	exit(0);
}

/* 
 * Load input data in binary.
 */
void
binary_load()
{
	register long insize, size;
	register unsigned int usize;
	register unsigned char *bp;

	if (fseek(ifp, 0L, 2) == -1)
		fatal("seek to end of input file failed");
	insize = ftell(ifp);
	if (fseek(ifp, 0L, 0) == -1)
		fatal("seek to start of input file failed");
	if ((inbuf = bp = lmalloc(insize)) == NULL)
		fatal("buffer allocation failed");
	inbufend = inbuf + insize;
	for (size = insize; size > 0L; ) {
		usize = (size > MAX_UINT) ? MAX_UINT : size;
		if (fread(bp, 1, usize, ifp) != usize)
			fatal("read error");
		bp += usize;
		size -= usize;
	}
	if (fclose(ifp) == EOF)
		fatal("cannot close input file");
}

void
fatal(s) char *s;
{
	fprintf(stderr, "put: %r\n", &s);
	exit(1);
}
