/*
 * tone.c
 * 7/17/89
 * Dump a D-10 tone.
 * Dump partials in keyboard edit order, not address order.
 */

#include <stdio.h>

#define	TSIZE	0xF6			/* tone size */
#define	PSIZE	0x3A			/* partial size */

char *kf[17] = { "-1", "-1/2", "-1/4", "0", "1/8", "1/4", "3/8", "1/2",
		 "5/8", "3/4", "7/8", "1", "5/4", "3/2", "2", "s1", "s2" };
char *pitch[12]= { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
int ispcm[13] = { 0x0, 0x0, 0x5, 0x5, 0xA, 0xF, 0xF, 0x0, 0xF, 0x0, 0x5, 0xA, 0xF };

extern char *bias();

unsigned char	buf[TSIZE];
unsigned char	name[128];

main(argc, argv) int argc; char *argv[];
{
	register int i;

	if (argc == 1) {

#if	1
		for (i = 0; i < 128; i++) {
			sprintf(name, "itone%03d", i);
			printf("Tone %c%02d timbre=%c%c%c midi=%03d  ",
				i < 64 ? 'a' : 'b',
				(i % 64) + 1,
				i < 64 ? 'A' : 'B',
				((i % 64) / 8) + '1',
				(i % 8) + '1',
				i);
			tone(name);
		}
#endif
#if	0
		for (i = 0; i < 64; i++) {
			sprintf(name, "tone%02d", i);
			printf("Tone i%02d                      ", i+1);
			tone(name);
		}
#endif
#if	0
		for (i = 0; i < 64; i++) {
			sprintf(name, "rtone%02d", i);
			printf("Tone r%02d                      ", i+1);
			tone(name);
		}
#endif
	} else
		for (i = 1; i <= argc - 1; i++)
			tone(argv[i]);
	exit(0);
}

tone(s) char *s;
{
	register unsigned char *bp;
	register FILE *fp;
	register int c, i, mute, pcm;
	int s12, s34;

	if ((fp = fopen(s, "rb")) == NULL) {
		printf("open error on \"%s\"\n", s);
		exit(1);
	} else if (fread(buf, TSIZE, 1, fp) != 1) {
		printf("read error\n");
		exit(1);
	} else if ((c = getc(fp)) != EOF) {
		printf("expected EOF but read %x\n", c);
		exit(1);
	} else if (fclose(fp) == EOF) {
		printf("close error\n");
		exit(1);
	}

	/* Common. */
	printf("\"");
	for (i = 0, bp = buf; i < 10; i++)
		printf("%c", *bp++);
	printf("\"\t");
	s12 = buf[10];
	s34 = buf[11];
	mute = buf[12];
	printf("s12=%2d\t", s12+1);
	if (mute & 0xC)
		printf("s34=%2d", s34+1);
	printf("\tmute=%1x\t%s\n\n", mute, buf[13]==0 ? "normal" : "no sustain");

	/* One to four partials. */
	for (i = c = 1; i <= 4; i++, c <<= 1) {
		bp = (i == 1) ? &buf[0x0E] : bp + PSIZE;
		pcm = (i <= 2) ? (c & ispcm[s12]) : (c & ispcm[s34]);
		if ((mute & c) == 0)
			continue;

		printf("Partial %d (%s):\n", i, (pcm) ? "PCM" : "Synth");

		printf("WG pitch/mod:\t");
		printf("%s%d\t", pitch[bp[0x00] % 12], bp[0x00]/12 + 1);
		printf("fine=%d\tkf=%s\t", bp[0x01]-50, kf[bp[0x02]]);
		if (bp[0x14])
			printf("LFOr=%d\tLFOd=%d\tLFOm=%d\t", bp[0x14], bp[0x15], bp[0x16]);
		else
			printf("No LFO\t\t\t");
		printf("bend=%s\n", (bp[0x03] == 1) ? "on" : "off");

		printf("WG form:\t");
		if (pcm)
			printf("\tbank=%d\twav=%d\t\t\t", bp[0x04]/2+1, bp[0x05]+1);
		else {
			printf("%s\t\t\t", (bp[0x04] & 1) ? "SAW" : "SQU");
			printf("pw=%d\tpwvs=%d\t", bp[0x06], bp[0x07]-7);
		}
		if (bp[0x08] == 0)
			printf("No PENV\n");
		else {
			printf("envd=%d\tevs=%d\tekf=%d\n", bp[0x08], bp[0x09], bp[0x0A]);
			printf("WG Penv:\t");
			printf("T1=%d\tT2=%d\tT3=%d\tT4=%d\t",
				bp[0x0B], bp[0x0C], bp[0x0D], bp[0x0E]);
			printf("L0=%d\tL1=%d\tL2=%d\tEndL=%d\n",
				bp[0x0F]-50, bp[0x10]-50, bp[0x11]-50, bp[0x13]-50);
		}

		if (!pcm) {
			printf("TVF freq:\t");
			printf("cut=%d\tres=%d\tkf=%s\t", bp[0x17], bp[0x18], kf[bp[0x19]]);
			if (bp[0x1B] == 7)
				printf("No bias\t\t");
			else
				printf("b=%s\tlev=%d\t", bias(bp[0x1A]), bp[0x1B]-7);
			printf("d=%d\tvs=%d\tdfk=%d\n", bp[0x1C], bp[0x1D], bp[0x1E]);

			printf("TVF env:\t");
			printf("Tkf=%d\t", bp[0x1f]);
			printf("T1=%d\tT2=%d\tT3=%d\tT4=%d\t",
				bp[0x20], bp[0x21], bp[0x22], bp[0x24]);
			printf("L1=%d\tL2=%d\tSusL=%d\n",
				bp[0x25], bp[0x26], bp[0x28]);
		}

		printf("TVA level:\t");
		printf("lev=%d\tvs=%d\t", bp[0x29], bp[0x2A]-50);
		if (bp[0x2C] == 12)
			printf("No b1\t\t");
		else
			printf("b1=%s\tL1=%d\t", bias(bp[0x2B]), bp[0x2C]-12);
		if (bp[0x2E] == 12)
			printf("No b2\t\t");
		else
			printf("b2=%s\tL2=%d\t", bias(bp[0x2D]), bp[0x2E]-12);
		printf("Tvf=%d\n", bp[0x30]);

		printf("TVA env:\t");
		printf("Tkf=%d\t", bp[0x2F]);
		printf("T1=%d\tT2=%d\tT3=%d\tT4=%d\t",
			bp[0x31], bp[0x32], bp[0x33], bp[0x35]);
		printf("L1=%d\tL2=%d\tSusL=%d\n",
			bp[0x36], bp[0x37], bp[0x39]);

		putchar('\n');
	}
	printf("-------------------------------------------------------------------------------\n");
}

char *
bias(n) register int n;
{
	static char buf[5];
	register char *bp, *pp;

	bp = &buf[0];
	if (n > 63) {
		*bp++ = '>';
		n -= 64;
	} else
		*bp++ = '<';
	n += 9;			/* reduce to normal pitch range */
	pp = pitch[n % 12];
	*bp++ = *pp++;
	if (*pp)
		*bp++ = *pp++;
	*bp++ = '1' + (n / 12);
	*bp = '\0';
	return &buf[0];
}
